package mySweetFactory;


import util.ContextChecker;



public class ConsoleImpl implements IConsole {

	
	/**
	 * Constant representing working in the heap.
	 */
	public static final int HEAP_CONTEXT = 1;

	/**
	 * Constant representing working in immortal.
	 */
	public static final int IMMORTAL_CONTEXT = 2;

	/**
	 * Constant representing working in a scope.
	 */
	public static final int SCOPE_CONTEXT = 4;

	/**
	 * Constant representing working on a java.lang.Thread
	 */
	public static final int SE_THREAD = 8;

	/**
	 * Constant representing working on a javax.realtime.RealtimeThread
	 * or a javax.realtime.NoHeapRealtimeThread
	 */
	public static final int RT_THREAD = 16;

	/**
	 * Constant representing working on a javax.realtime.NoHeapRealtimeThread
	 */
	public static final int NHRT_THREAD = 32;


	public static final long CALL_SNOOZE_TIME_MILLIS = 3000;
	
	
	
	public void report(String measurement) {
		
		ContextChecker.checkContext(SCOPE_CONTEXT + NHRT_THREAD);
		
		//System.out.println("[Console] Logging the measurement: " + measurement.getData());
		System.out.println("[Console] Logging the measurement: " + measurement);
		
		
		snooze();
	}
	
	
	
	

	private void snooze()
	{
		try {
			Thread.sleep(CALL_SNOOZE_TIME_MILLIS);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}


}
