package mySweetFactory;

import java.util.ArrayList;
import java.util.List;

import javax.realtime.LTMemory;

import mySweetFactory.measurement.Measurement;
import mySweetFactory.measurement.MeasurementManager;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;

import util.ContextChecker;
import util.MemoryUsagePrint;
import util.Print;
import util.PrintMessage;




public class ProductionLineImpl implements Runnable, BindingController {

	
	public static final long WAIT_TIME_MILLIS = 10;
	
	private volatile boolean quit = false;
	
    /**
     * 
     * 
     */
	IMonitor iMonitor;
	
	MeasurementManager measurementManager;
	
	
	
	/**
	 * RT-technical attributes
	 */
	static MemoryUsagePrint printUsage;
	static Print printData;
	static PrintMessage printMessage; 
	static LTMemory printingScope;
	
	
	/**
	 * 
	 * 
	 * 
	 * 
	 * @param pp
	 * @param rp
	 */
	public ProductionLineImpl(){
		//super();
		//System.out.println("[Producer] initialization.");
		
		printUsage = new MemoryUsagePrint();
		printingScope =  new LTMemory(4*1024); 
		printUsage.setCallerName("ProductionLine");
		
		printData = new Print();
		printData.setCallerName("ProductionLine");
		
		printMessage = new PrintMessage();
		printMessage.setCallerName("ProductionLine");
		
		
		
		
	}

	public void initialize(MeasurementManager manager) {
		measurementManager = manager;
	}


	public void run(){

		printMessage.message = "running....";
		printingScope.enter(printMessage);
		
		
		ContextChecker.checkContext(ContextChecker.IMMORTAL_CONTEXT);
		ContextChecker.checkContext(ContextChecker.NHRT_THREAD);
		
		
		
		for (int i = 0; i < 5;i++) {
			//String measurement = new String("HelloWorld, message n." + i); 
			Measurement measurement = measurementManager.getMeasurement(); 
			iMonitor.processMeasurement(measurement);
			
			try {
				Thread.sleep(WAIT_TIME_MILLIS);
			} catch (InterruptedException e) {
				//Can't easily do anything with this for fear of leaking immortal.
				//If this was a likely possible problem we could create a runnable to 
				//handle it in a scope.
			}
			
		}
		
		
		
		
		
		printMessage.message = "end....";
		printingScope.enter(printMessage);
	}


	public void quit(){
		quit = true;
	}


	
	

	public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException {
         if (clientItfName.equals("iMonitor")) {
            if (!(IMonitor.class.isAssignableFrom(serverItf.getClass()))) {
                throw new IllegalBindingException(((("server interfaces connected to " + clientItfName) + " must be instances of ") + (IMonitor.class.getName())));
            } 
            iMonitor = ((IMonitor)(serverItf));
            return ;
        } 
        
                  
         
        throw new NoSuchInterfaceException((("Client interface \'" + clientItfName) + "\' is undefined."));
    }
    
    public String[] listFc() {
        List __interfaces__ = new ArrayList();
        __interfaces__.add("iMonitor");
        
        return ((String[])(__interfaces__.toArray(new String[]{  })));
    }
    
    public Object lookupFc(String clientItfName) throws NoSuchInterfaceException {
       
        if (clientItfName.equals("iMonitor")) {
            return  iMonitor;
        } 
      
        
        throw new NoSuchInterfaceException((("Client interface \'" + clientItfName) + "\' is undefined."));
    }
    
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException {
        
        
        if (clientItfName.equals("iMonitor")) {
        	iMonitor = null;
           return ;
       } 
        
      
        
        throw new NoSuchInterfaceException((("Client interface \'" + clientItfName) + "\' is undefined."));
    }
	
	
	
	
	
	
	
	
	
	
	
	
}
