/*
 * DeveloperWorks RTJ Article 5 SweetFactory Demo
 * (C) Copyright IBM Corp. 2007. All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 */

package util;

import javax.realtime.MemoryArea;
import javax.realtime.PriorityParameters;
import javax.realtime.PriorityScheduler;
import javax.realtime.RealtimeThread;




/**
 * Utility class to help with bootstrapping objects into the correct memory areas.
 * 
 * @author <a href="mailto:andhall@uk.ibm.com">Andrew Hall</a>
 *
 */
public class Bootstrapper {

	
	/**
	 * Constant representing working in the heap.
	 */
	public static final int HEAP_CONTEXT = 1;

	/**
	 * Constant representing working in immortal.
	 */
	public static final int IMMORTAL_CONTEXT = 2;

	/**
	 * Constant representing working in a scope.
	 */
	public static final int SCOPE_CONTEXT = 4;

	/**
	 * Constant representing working on a java.lang.Thread
	 */
	public static final int SE_THREAD = 8;

	/**
	 * Constant representing working on a javax.realtime.RealtimeThread
	 * or a javax.realtime.NoHeapRealtimeThread
	 */
	public static final int RT_THREAD = 16;

	/**
	 * Constant representing working on a javax.realtime.NoHeapRealtimeThread
	 */
	public static final int NHRT_THREAD = 32;

	
	/**
	 * Runs the supplied memory area in the supplied memory area.
	 * 
	 * Is designed to do this from a java.lang.Thread (which can't call enter()
	 * on MemoryAreas) and works by spawning a short-lived realtime thread
	 * to do the work and waiting for it to complete.
	 * 
	 * @param area Memory area to work in
	 * @param r Runnable to execute.
	 */
	public static void runInArea(MemoryArea area,Runnable r){
		ContextChecker.checkContext(HEAP_CONTEXT + SE_THREAD);
		
		RealtimeThread t = new RealtimeThread(new PriorityParameters(PriorityScheduler.MIN_PRIORITY),
											null,
											null,
											area,
											null,
											r);
		
		t.start();
		
		try {
			t.join();
		} catch (InterruptedException e) {
			throw new RuntimeException("Unexpected interruption",e);
		}
		
	}
	
}
