
package controllers;

import java.lang.reflect.Modifier;

import org.objectweb.fractal.adl.juliac.MembraneDesc;
import org.objectweb.fractal.adl.juliac.proxy.InterceptorSourceCodeGenerator;
import org.objectweb.fractal.adl.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.adl.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.adl.juliac.visit.ClassSourceCodeVisitor;
import org.objectweb.fractal.adl.juliac.visit.FileSourceCodeVisitor;
import org.objectweb.fractal.adl.juliac.visit.MethodSourceCodeVisitor;
import org.objectweb.fractal.api.type.InterfaceType;

//import spoon.reflect.Factory;

/**
 * This class generates the source code of lifecycle interceptors.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class SoleilSourceCodeGenerator extends InterceptorSourceCodeGenerator {
        
    public SoleilSourceCodeGenerator() {
        super();
    }
    
    public SoleilSourceCodeGenerator(
        InterfaceType it, MembraneDesc membraneDesc, boolean mergeable ) {
        
        super(it, membraneDesc, mergeable );
    }
    
    /**
     * Return the name of the suffix which is appended to generated class names.
     */
    public String getClassNameSuffix() {
        return "SoleilProducer";
    }
    
    /**
     * Return true if some code must be generated by the current source code
     * generator.
     */
//    /@Override
    public boolean match() {
        if( it.isFcClientItf() ) {
            return false;   // Interceptors don't match on client interfaces
        }
        String name = it.getFcItfName();
        if( name.startsWith("//") ) {
            return false;    // Interceptors don't match on Koch meta interfaces
        }
        if( name.endsWith("-controller") || name.equals("component") ) {
            return false;   // Interceptors don't match on control interfaces
        }
        
     // Interceptors match only for server interfaces whose names end with "Runnable"
        if( name.endsWith("Runnable")  ) {
            return true;   
        }
        else 
        	return false;
        
    }

    public void generateImports( FileSourceCodeVisitor fv ) {
    	
    	fv.visitImport("javax.realtime.ImmortalMemory");
    	fv.visitImport("javax.realtime.NoHeapRealtimeThread");
    	fv.visitImport("util.PrioritySpecifications");
    	
    }
    
    
    
    /**
     * Return the name of the class implementing the active controller.
     */
    /*protected String getACCtrlImplName() {
        
        Juliac jc = Juliac.get();
        Factory factory = jc.getSpoonFactory();
        ClassLoader classloader = jc.getClassLoader();

        List<ControllerDesc> ctrlDescs = membraneDesc.getCtrlDescs();
        UnifiedClass<ActiveController> acCtrlUc =
            new UnifiedClass<ActiveController>(
                factory,classloader,ActiveController.class);
        String acCtrlImplName = null;
        for (ControllerDesc ctrlDesc : ctrlDescs) {
            String ctrlImplName = ctrlDesc.getImplName();
            UnifiedClass<?> ctrlImpl =
                new UnifiedClass<Object>(factory,classloader,ctrlImplName);
            if( acCtrlUc.isAssignableFrom(ctrlImpl) ) {
                acCtrlImplName = ctrlImplName;
            }
        }
        
        if( acCtrlImplName == null ) {
            String msg = "No active controller";
            throw new IllegalArgumentException(msg);
        }
        
        return acCtrlImplName;
    }*/
    
    /**
     * Return the name of the class implementing the mailbox.
     */
    /*protected String getMailBoxCtrlImplName() {
        
        Juliac jc = Juliac.get();
        Factory factory = jc.getSpoonFactory();
        ClassLoader classloader = jc.getClassLoader();

        List<ControllerDesc> ctrlDescs = membraneDesc.getCtrlDescs();
        UnifiedClass<MailBox> mbCtrlUc =
            new UnifiedClass<MailBox>(
                factory,classloader,MailBox.class);
        String mbCtrlImplName = null;
        for (ControllerDesc ctrlDesc : ctrlDescs) {
            String ctrlImplName = ctrlDesc.getImplName();
            UnifiedClass<?> ctrlImpl =
                new UnifiedClass<Object>(factory,classloader,ctrlImplName);
            if( mbCtrlUc.isAssignableFrom(ctrlImpl) ) {
                mbCtrlImplName = ctrlImplName;
            }
        }
        
        if( mbCtrlImplName == null ) {
            String msg = "No MailBox";
            throw new IllegalArgumentException(msg);
        }
        
        return mbCtrlImplName;
    }*/
        
    //@Override
    public String[] getImplementedInterfaceNames() {
        return new String[]{it.getFcItfSignature()};
    	//return null;
    }
    
    //@Override
    public void generateFields( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {
        if( mergeable ) {
            //cv.visitField(Modifier.PUBLIC, "int", "_this_fcState", null);
            //cv.visitField(Modifier.PUBLIC, "int", "_this_fcInvocationCounter", null);
        }
        else {
        	// Nothing for this basic example...
        	cv.visitField(Modifier.PRIVATE, "javax.realtime.NoHeapRealtimeThread", "_nhrt", null);
        }
    }
        
    //@Override
    public void generateStaticParts( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {
    }

    //@Override
    public void generateConstructors( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {
    }

    //@Override
    protected void generateMethodInitFcController( MethodSourceCodeVisitor mv, UnifiedClass proxycl ) {

/*        String acCtrlImplName = getACCtrlImplName();           
        String mbCtrlImplName = getMailBoxCtrlImplName();

        // Retrieve the reference to the active controller
        mv.visitln("    Object oactrl = ic.getInterface(\"active-controller\");");
        mv.visit  ("    if( ! (oactrl instanceof ");
        mv.visit  (acCtrlImplName);
        mv.visitln(") ) {");
        mv.visit  ("      while( oactrl instanceof ");
        mv.visit  (Interceptor.class.getName());
        mv.visitln(" ) {");
        mv.visit  ("        oactrl = ((");
        mv.visit  (Interceptor.class.getName());
        mv.visitln(")oactrl).getFcItfDelegate();");
        mv.visitln("      }");
        mv.visitln("    }");
        
        // Record the reference to the instance implementing the controller
        mv.visit  ("    _actrl = (");
        mv.visit  (acCtrlImplName);
        mv.visitln(") oactrl;");
        
        // Retrieve the reference to the mailbox
        mv.visitln("    Object ombctrl = ic.getInterface(\"mailbox-internal-controller\");");
        mv.visit  ("    if( ! (ombctrl instanceof ");
        mv.visit  (mbCtrlImplName);
        mv.visitln(") ) {");
        mv.visit  ("      while( ombctrl instanceof ");
        mv.visit  (Interceptor.class.getName());
        mv.visitln(" ) {");
        mv.visit  ("        ombctrl = ((");
        mv.visit  (Interceptor.class.getName());
        mv.visitln(")ombctrl).getFcItfDelegate();");
        mv.visitln("      }");
        mv.visitln("    }");
        
        // Record the reference to the instance implementing the controller
        mv.visit  ("    _mbctrl = (");
        mv.visit  (mbCtrlImplName);
        mv.visitln(") ombctrl;");
*/
    	
    	//mv.visitln("	_nhrt = new NoHeapRealtimeThread(null, null, null, ImmortalMemory.instance(), null, _impl);");
    	
    	mv.visitln("	_nhrt = new NoHeapRealtimeThread(" +
			"PrioritySpecifications.PRODUCER_THREAD_PRIORITY, null, null,ImmortalMemory.instance(), null, _impl);");

    	
    }
        
    //@Override
    protected void generateMethodClone( MethodSourceCodeVisitor mv, UnifiedClass proxycl ) {
        //mv.visitln("    clone._actrl = _actrl;");
        //mv.visitln("    clone._mbctrl = _mbctrl;");
    }
    
    //@Override
    protected void generateMethodOthers( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {
    	
    	//super.generateMethodOthers (cv, proxycl);
/*    	String params[] = {"controllers.Message msg"};
    	MethodSourceCodeVisitor mv =
    		cv.visitMethod(Modifier.PRIVATE, null, "void", "proceed", params, null );
    	mv.visitBegin();
    	mv.visitln("    // for the moment, the proceed is an hard-written generation...");
    	mv.visitln("    if (msg.getMethodName().equals(\"mA\")) {");
    	mv.visitln("         _impl.mA(null);");
    	mv.visitln("         msg.setResultObject((null));");
    	mv.visitln("    }");
    	mv.visitln("    else {");
    	mv.visitln("         _impl.mB(0, 0);");
    	mv.visitln("         msg.setResultObject(null);");
    	mv.visitln("    }");
    	mv.visitEnd();
    	
    	// Method used by the active controller
    	mv = cv.visitMethod(Modifier.PUBLIC, null, "void", "Execute", null, null );
    	mv.visitBegin();
    	mv.visitln("    System.err.println(\"Execute invocation in the active interceptor\" + \", Thread Id : \" + Thread.currentThread().getId());");
    	mv.visitln("    System.err.println(\"Retrieve from mailbox the informations needed\");");
    	mv.visitln("    controllers.Message msg = _mbctrl.getFirstMessage();");
    	mv.visitln("    proceed (msg);");
    	mv.visitln("    return;");
    	mv.visitEnd();
*/
    }

    //@Override
    public void generateProxyMethodBodyBeforeCode(
        MethodSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
    	
        if( mergeable ) {
        }
        else {
  
        	mv.visitln("  // Soleil code - before");
        	mv.visitln("	if (true) {");
        	mv.visitln("    	  System.out.println(\"[SoleilInterceptor]: starting nhrt thread.\");");
        	mv.visitln("    	  _nhrt.start();");
            mv.visitln("      	return;");
            mv.visitln("	}");
        	
        	
        }
        //mv.visitln("      }");
        //mv.visitln("      try {");
    }
    
    //@Override
    public void generateProxyMethodBodyAfterCode(
    	MethodSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
    	
    	if (mergeable) {
    	}
    	else {
    		/*
            mv.visitln("      }");
            mv.visitln("      System.err.println(\"waiting for results / synchronous only\");");
            mv.visitln("      Object result = msg.getResultObject();");
            mv.visitln("      System.err.println(\"return the active interceptor\" + \", Thread Id : \" + Thread.currentThread().getId());");
            */
        	mv.visitln("// Soleil code - after");
    	}
    }
    
}
