package mySweetFactory;



//import static util.ContextChecker.IMMORTAL_CONTEXT;
//import static util.ContextChecker.NHRT_THREAD;

import java.util.ArrayList;
import java.util.List;

import javax.realtime.LTMemory;
import javax.realtime.RealtimeThread;
import javax.realtime.WaitFreeWriteQueue;

import mySweetFactory.audit.IAudit;
import mySweetFactory.measurement.Measurement;
import mySweetFactory.measurement.MeasurementManager;
import mySweetFactory.util.ContextChecker;
import mySweetFactory.util.MemoryUsagePrint;
import mySweetFactory.util.Print;
import mySweetFactory.util.PrintMessage;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;

/**
 * 
 * 
 * @author Ales
 *
 */
public class MonitoringSystemImpl  implements Runnable, IMonitor, BindingController {
  
	/**
	 * Constant representing working in the heap.
	 */
	//public static final int HEAP_CONTEXT = 1;

	/**
	 * Constant representing working in immortal.
	 */
	public static final int IMMORTAL_CONTEXT = 2;

	/**
	 * Constant representing working in a scope.
	 */
	//public static final int SCOPE_CONTEXT = 4;

	/**
	 * Constant representing working on a java.lang.Thread
	 */
	//public static final int SE_THREAD = 8;

	/**
	 * Constant representing working on a javax.realtime.RealtimeThread
	 * or a javax.realtime.NoHeapRealtimeThread
	 */
	//public static final int RT_THREAD = 16;

	/**
	 * Constant representing working on a javax.realtime.NoHeapRealtimeThread
	 */
	//public static final int NHRT_THREAD = 32;
	
	/**
   *   
   *  Component Attributes
   */
	private WaitFreeWriteQueue iConsumer;
	
	
	
	/**
	 * RT-technical attributes
	 */
	static MemoryUsagePrint printUsage;
	static Print printData;
	static PrintMessage printMessage; 
	static LTMemory printingScope;
	private volatile boolean quit = false;
	
	
	/**
	 * 
	 * Fractal Interface binding with the Console component
	 * 
	 */
	IConsole iConsole;
	
  IAudit iAudit;
	
	MeasurementManager measurementManager;
  
  
	
	/**
	 * 
	 * 
	 * 
	 * 
	 * @param pp
	 * @param rp
	 * @param q
	 */
	public MonitoringSystemImpl(){
		super();
	}
	


	
	
  /**
   * 
   * 
   * 
   * @param productionQueue
   */
	public void initialize(WaitFreeWriteQueue productionQueue, MeasurementManager manager) {

		iConsumer = productionQueue;
		
		printUsage = new MemoryUsagePrint();
		printingScope =  new LTMemory(4*1024); 
		printUsage.setCallerName("MonitoringSystem");
		
		printData = new Print();
		printData.setCallerName("MonitoringSystem");
		
		printMessage = new PrintMessage();
		printMessage.setCallerName("MonitoringSystem");
		
		measurementManager = manager;
		
	}
	
	
	
	/**
	 * 
	 * 
	 * 
	 * 
	 * 
	 */
	public void run(){
		
		
		printMessage.message = "running....";
		printingScope.enter(printMessage);
		
		
		ContextChecker.checkContext(ContextChecker.IMMORTAL_CONTEXT);
		ContextChecker.checkContext(ContextChecker.NHRT_THREAD);
		
		
		
		try {
			
			
			
			for (int i = 0; i < ProductionLineImpl.ITERATIONS; i++) {

				
				Measurement measurement = (Measurement) iConsumer.read();
				memConsumed();
				
				iConsole.report(measurement.getData());
				
				memConsumed();
				
				//TODO: some pool of objects
				//
				//iAudit.log("Message to Audit: " + "Measurement n."
				//		+ measurement.getData());
        //
					
				measurementManager.returnMeasurement(measurement);
					
			}
			
			
			
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		
		
		
				
		
		
		
		memConsumed();
		printMessage.message = " end....";
		printingScope.enter(printMessage);
		
		
		
	}


  /**
   * 
   * 
   * 
   * @param measurement
   */
	public void processMeasurement(Measurement measurement) {
		printData.printData = measurement;
		printingScope.enter(printData);
		printingScope.enter(printUsage);
		
		
		
	}



	
	
	
	
	
	
	/**
	 * 
	 * 
	 * 
	 * @return
	 */
	public WaitFreeWriteQueue getIConsumer() {
		return iConsumer;
	}
	
	
	

	/*
	 * Setters and Getters
	 * 
	 * 
	 */
	public void setIConsumer(WaitFreeWriteQueue consumer) {
		iConsumer = consumer;
	}








	public void quit() {
		quit = true;
		
		
	}

	
	
	
	
	public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException {
        if (clientItfName.equals("iConsole")) {
            if (!(IConsole.class.isAssignableFrom(serverItf.getClass()))) {
                throw new IllegalBindingException(((("server interfaces connected to " + clientItfName) + " must be instances of ") + (IConsole.class.getName())));
            } 
            iConsole = ((IConsole)(serverItf));
            return ;
        } 

        
        if (clientItfName.equals("iAudit")) {
          if (!(IAudit.class.isAssignableFrom(serverItf.getClass()))) {
              throw new IllegalBindingException(((("server interfaces connected to " + clientItfName) + " must be instances of ") + (IConsole.class.getName())));
          } 
          iAudit = ((IAudit)(serverItf));
          return ;
      } 
       
        
        throw new NoSuchInterfaceException((("Client interface \'" + clientItfName) + "\' is undefined."));
    }
    
    public String[] listFc() {
        List __interfaces__ = new ArrayList();
        __interfaces__.add("iConsole");
        __interfaces__.add("iAudit");
        return ((String[])(__interfaces__.toArray(new String[]{  })));
    }
    
    public Object lookupFc(String clientItfName) throws NoSuchInterfaceException {
        if (clientItfName.equals("iConsole")) {
            return  iConsole;
        } 
      
        if (clientItfName.equals("iAudit")) {
          return  iAudit;
      } 
        
        throw new NoSuchInterfaceException((("Client interface \'" + clientItfName) + "\' is undefined."));
    }
    
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException {
        if (clientItfName.equals("iConsole")) {
        	 iConsole = null;
            return ;
        } 
        
        if (clientItfName.equals("iAudit")) {
        	iAudit = null;
           return ;
       } 
        
        throw new NoSuchInterfaceException((("Client interface \'" + clientItfName) + "\' is undefined."));
    }
	
	
	
	
	
	
	
	

    /**
     * 
     * 
     */
  	private void memConsumed() {
  		printingScope.enter(printUsage);
  	}
  	
	
  	 /**
     * 
     * 
     */
  	private void memConsumed(String output) {
  		printUsage.message = output;
  		printingScope.enter(printUsage);
  	}
  	
	
	

	/**
	 * Launching the MonitoringSystem logic in the ImmortalMemory
	 */
//	public void main(String[] args) {
//		ImmortalMemory.instance().enter(this);
//		
//	}


	
	

}
